package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesConfigService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesNumberRuleMatchDispatchService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProduceSnExtService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesPullingOrderInfoService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;


@Service
@Slf4j
public class MesPullingOrderInfoService implements IMesPullingOrderInfoService {
    public static final Logger LOGGER = LoggerFactory.getLogger(MesPullingOrderInfoService.class);
    @Autowired
    private MesPullingOrderInfoRepository mesPullingOrderInfoRepository;
    @Autowired
    private MesPullingOrderPartInfoRepository mesPullingOrderPartInfoRepository;
    @Autowired
    private MesPartPullRepository mesPartPullRDao;
    @Autowired
    private MesPartPullDetailRepository mesPartPullDetailRDao;
    @Autowired
    private MesWorkOrderPartRepository workOrderPartRepository;

    @Autowired
    private MesWorkCenterRepository mesWorkCenterRepository;
    @Autowired
    private MesPullingOrderExtendsRepository mesPullingOrderExtendsRepository;

    @Autowired
    private IMesProduceSnExtService mesProduceSnExtService;

    @Autowired
    private IMesConfigService configService;
    @Override
    public ListPager<MesPullingOrderInfo> queryMesPullingOrderInfoByPager(MesPullingOrderInfo bean, Pager pager) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        setPackQueryBean(bean, packBean);
        if (!StringUtil.isEmpty(bean.orderBy())) {
            packBean.setOrderByStr(bean.orderBy());
        } else {
            DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
        }
        //
        if (!StringUtil.isEmpty(bean.getCreateDatetime())) {
            DdlPreparedPack.getStringBiggerPack(bean.getCreateDatetime(), "createDatetime", packBean);
        }
        pager = PagerHelper.getPager(pager, mesPullingOrderInfoRepository.findByHqlWhereCount(packBean));

        List<MesPullingOrderInfo> resultList = mesPullingOrderInfoRepository.findByHqlWherePage(packBean, pager);

        return new ListPager<>(resultList, pager);
    }

    protected void setPackQueryBean(MesPullingOrderInfo bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getPullCode(), "pullCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getPullingOrderNo(), "pullingOrderNo", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getPullOrderType(), "pullOrderType", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getPrintStatus(), "printStatus", packBean);
        //已确认、已扫描、已发送设备
        if (StringUtil.isEmpty(bean.getPullOrderStatus())) {
            DdlPreparedPack.getInPackList(Arrays.asList(
                    MesExtEnumUtil.PULL_ORDER_STATUS.KITTING_PULL.getValue(),
                    MesExtEnumUtil.PULL_ORDER_STATUS.JIS_PULL.getValue(),
                    MesExtEnumUtil.PULL_ORDER_STATUS.SPS_PULL.getValue()), "pullOrderStatus", packBean);
        } else {
            DdlPreparedPack.getNumEqualPack(bean.getPullOrderStatus(), "pullOrderStatus", packBean);
        }
        DdlPreparedPack.getStringEqualPack(bean.getWorkOrderNo(), "workOrderNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
    }

    @Override
    public void doMesPullingOrderInfoScan(MesPullingOrderInfo bean, String userName) {
        //校验是不是第一行数据
        DdlPackBean packBeanOne = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(bean.getPullCode(), "pullCode", packBeanOne);
        DdlPreparedPack.getStringEqualPack(bean.getPullingOrderNo(), "pullingOrderNo", packBeanOne);
        DdlPreparedPack.getStringEqualPack(bean.getWorkCenterCode(), "workCenterCode", packBeanOne);
//        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.PULL_ORDER_STATUS.KITTING_PULL.getValue(), "pullOrderStatus", packBeanOne);
        MesPullingOrderInfo mesPullingOrderInfo = mesPullingOrderInfoRepository.getByProperty(packBeanOne);
        if (StringUtil.isEmpty(mesPullingOrderInfo)) {
            MesPcnException.throwMesBusiException("扫描拉动单号【%s】查询数据为空", bean.getPullingOrderNo());
        }
        if (mesPullingOrderInfo.getPullOrderStatus() != MesExtEnumUtil.PULL_ORDER_STATUS.KITTING_PULL.getValue()) {
            MesPcnException.throwMesBusiException("扫描拉动单号【%s】拉动单状态不是【已确认】", bean.getPullingOrderNo());
        }

        //强过码 如果强过为true 不校验顺序 直接扫描完成修改状态为已扫描
        //扫完单号 扫强过码校验通过后 单号顺序之前的单子变成无序 都可扫描
//        String spsPass = configService.getCfgValue(bean.getOrganizeCode(), MesPcnExtConstWords.SPS_STRONGER_PASS);
        if (mesPullingOrderInfo.getIsPassCode()==CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()
                ||(!Objects.isNull(bean.getIsPass()) && CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()==bean.getIsPass())) {
            mesPullingOrderInfo.setPullOrderStatus(MesExtEnumUtil.PULL_ORDER_STATUS.JIS_PULL.getValue());
            ConvertBean.serviceModelUpdate(mesPullingOrderInfo, userName);
            mesPullingOrderInfoRepository.update(mesPullingOrderInfo);
            //修改当前单号之前的为可强过
            DdlPackBean packBeanPass= DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(bean.getPullCode(), "pullCode", packBeanPass);
            DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.PULL_ORDER_STATUS.KITTING_PULL.getValue(), "pullOrderStatus", packBeanOne);
            DdlPreparedPack.getStringSmallerNotEqualPack(bean.getPullingOrderNo(), "pullingOrderNo", packBeanPass);
            DdlPreparedPack.getStringEqualPack(bean.getWorkCenterCode(), "workCenterCode", packBeanPass);
            List<MesPullingOrderInfo> mesPullingOrderInfos = mesPullingOrderInfoRepository.findByHqlWhere(packBeanPass);
            if (!CollectionUtils.isEmpty(mesPullingOrderInfos)){
                for (MesPullingOrderInfo pullingOrderInfo : mesPullingOrderInfos) {
                    pullingOrderInfo.setIsPassCode(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                    ConvertBean.serviceModelUpdate(pullingOrderInfo, userName);
                }
                mesPullingOrderInfoRepository.saveAll(mesPullingOrderInfos);
            }
        } else {
            DdlPackBean packBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(bean.getPullCode(), "pullCode", packBean);
            DdlPreparedPack.getStringEqualPack(bean.getWorkCenterCode(), "workCenterCode", packBean);
            DdlPreparedPack.getNumEqualPack(bean.getPullOrderType(), "pullOrderType", packBean);
            DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.PULL_ORDER_STATUS.KITTING_PULL.getValue(), "pullOrderStatus", packBean);

            //顺序校验 根据未扫描单号 1 、2 、3，如果没有强过码 必须先扫1 再扫2  pullingOrderNo修改位workOrderNo
            DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.ASC.getValue()}, new String[]{"pullingOrderNo"}, packBean);
            List<MesPullingOrderInfo> pullingOrderInfos = mesPullingOrderInfoRepository.findByHqlWhere(packBean);
            if (!CollectionUtils.isEmpty(pullingOrderInfos)) {
                //按照排序 依次扫描，如果扫描不按照顺序 报错
                MesPullingOrderInfo pullingOrderInfo = pullingOrderInfos.get(0);
                if (pullingOrderInfo.getPullingOrderNo().equals(mesPullingOrderInfo.getPullingOrderNo())) {
                    mesPullingOrderInfo.setPullOrderStatus(MesExtEnumUtil.PULL_ORDER_STATUS.JIS_PULL.getValue());
                    ConvertBean.serviceModelUpdate(mesPullingOrderInfo, userName);
                    mesPullingOrderInfoRepository.update(mesPullingOrderInfo);
                } else {
                    MesPcnException.throwMesBusiException("扫描拉动单号【%s】SPS排序单校验顺序错误", bean.getPullingOrderNo());
                }
            } else {
                MesPcnException.throwMesBusiException("扫描拉动单号产线拉动组【%s】数据为空", bean.getPullingOrderNo());
            }
        }

    }

    @Override
    public List<MesPullingOrderInfo> doMesPullingOrderInfoPrint(MesPullingOrderInfo bean, String userName) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        //多选
        if (!StringUtil.isEmpty(bean.getPullCode())) {
            DdlPreparedPack.getInPackList(Arrays.asList(bean.getPullCode().split(",")), "pullCode", packBean);
        }
        if (!StringUtil.isEmpty(bean.getWorkCenterCode())) {
            DdlPreparedPack.getInPackList(Arrays.asList(bean.getWorkCenterCode().split(",")), "workCenterCode", packBean);
        }
        DdlPreparedPack.getStringEqualPack(bean.getPullingOrderNo(), "pullingOrderNo", packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.PRINT_STATUS.UNPRINT.getValue(), "printStatus", packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
        List<MesPullingOrderInfo> pullingOrderInfos = mesPullingOrderInfoRepository.findByHqlWhere(packBean);
        if (!CollectionUtils.isEmpty(pullingOrderInfos)) {
            //把查出来的拉动单打印并修改打印状态为已打印
            for (MesPullingOrderInfo pullingOrderInfo : pullingOrderInfos) {
                //查询拉动组明细
                DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(pullingOrderInfo.getOrganizeCode());
                DdlPreparedPack.getStringEqualPack(pullingOrderInfo.getPullingOrderNo(), "pullingOrderNo", partPackBean);
                List<MesPullingOrderPartInfo> pullingOrderPartInfos = mesPullingOrderPartInfoRepository.findByHqlWhere(partPackBean);
                if (!CollectionUtils.isEmpty(pullingOrderPartInfos)) {
                    pullingOrderInfo.setPartCount(pullingOrderPartInfos.size());
                    pullingOrderInfo.setPullingOrderPartInfos(pullingOrderPartInfos);
                }

                pullingOrderInfo.setPrintStatus(MesExtEnumUtil.PRINT_STATUS.PRINTED.getValue());
                ConvertBean.serviceModelUpdate(pullingOrderInfo, userName);
            }
            mesPullingOrderInfoRepository.saveAll(pullingOrderInfos);
        }
        return pullingOrderInfos;
    }

    @Override
    public ListPager queryMesPullingOrderPartInfoByPager(MesPullingOrderInfo bean, Pager pager) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        if (!StringUtil.isEmpty(bean.getPullCode())) {
            DdlPreparedPack.getInPackList(Arrays.asList(bean.getPullCode().split(",")), "pullCode", packBean);
        }
        //查询拉动单主表是JIS拉动且拉动状态为10
        String append = packBean.getWhereAppend() +
                " AND EXISTS (select 1 from MesPullingOrderInfo po where po.pullingOrderNo = model.pullingOrderNo" +
                " and po.pullOrderType=20 and  po.workCenterCode ='" + bean.getWorkCenterCode() + "' ";
        packBean.setWhereAppend(append + " )");
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.ASC.getValue(), CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"status", "pullingOrderNo"}, packBean);

        pager = PagerHelper.getPager(pager, mesPullingOrderPartInfoRepository.findByHqlWhereCount(packBean));

        List<MesPullingOrderPartInfo> resultList = mesPullingOrderPartInfoRepository.findByHqlWherePage(packBean, pager);
        if (!CollectionUtils.isEmpty(resultList)) {
            //查拉动单明细对应的拉动组规则
            DdlPackBean packBeanDetail = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getInPackList(resultList.stream().map(MesPullingOrderPartInfo::getPullCode).collect(Collectors.toList()), "pullCode", packBeanDetail);
            List<MesPartPullDetail> mesPartPullDetails = mesPartPullDetailRDao.findByHqlWhere(packBeanDetail);
            if (!CollectionUtils.isEmpty(mesPartPullDetails)) {
                Map<String, List<MesPartPullDetail>> pullCodePartMap = mesPartPullDetails.stream().collect(Collectors.groupingBy(a -> a.getPullCode() + "=" + a.getPartNo()));
                for (MesPullingOrderPartInfo partInfo : resultList) {
                    if (!CollectionUtils.isEmpty(pullCodePartMap.get(partInfo.getPullCode() + "=" + partInfo.getPartNo()))) {
                        MesPartPullDetail mesPartPullDetail = pullCodePartMap.get(partInfo.getPullCode() + "=" + partInfo.getPartNo()).get(0);
                        partInfo.setScanValidationType(StringUtil.isEmpty(mesPartPullDetail.getScanValidationType()) ? 10 : mesPartPullDetail.getScanValidationType());
                        partInfo.setRuleValidation(StringUtil.isEmpty(mesPartPullDetail.getRuleValidation()) ? "" : mesPartPullDetail.getRuleValidation());
                        if (mesPartPullDetail.getScanValidationType() == MesExtEnumUtil.PART_PULL_DETAIL_SCAN_VALIDATION_TYPE.UN_VALIDATE.getValue()) {
                            partInfo.setColor(MesExtEnumUtil.PART_PULL_DETAIL_COLOR.WHITE.getCode());
                        }else if (partInfo.getColor()==MesExtEnumUtil.PART_PULL_DETAIL_COLOR.WHITE.getCode()){
                            partInfo.setColor(MesExtEnumUtil.PART_PULL_DETAIL_COLOR.RED.getCode());
                        }
                    }
                }
            }
        }
        return new ListPager<>(resultList, pager);
    }

    @Override
    public MesPullingOrderPartInfo doMesPullingOrderInfoJisScan(MesPullingOrderPartInfo mesPullingOrderInfo, String userName) {

        MesPullingOrderPartInfo partInfo = mesPullingOrderPartInfoRepository.getById(mesPullingOrderInfo.getId());
        if (!StringUtil.isEmpty(partInfo)) {
            //查拉动单明细对应的拉动组规则
            DdlPackBean packBeanDetail = DdlPackBean.getDdlPackBean(partInfo.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(partInfo.getPullCode(), "pullCode", packBeanDetail);
            DdlPreparedPack.getStringEqualPack(partInfo.getPartNo(), "partNo", packBeanDetail);
            MesPartPullDetail mesPartPullDetail = mesPartPullDetailRDao.getByProperty(packBeanDetail);
            if (StringUtil.isEmpty(mesPartPullDetail)) {
                MesPcnException.throwMesBusiException("工单【%s】拉动组数据为空", mesPullingOrderInfo.getWorkOrderNo());
            }
            //校验规则
            if (checkRule(mesPullingOrderInfo, mesPartPullDetail)) {
                //校验成功修改颜色为黄色2
                partInfo.setColor(MesExtEnumUtil.PART_PULL_DETAIL_COLOR.YELLOW.getCode());
                ConvertBean.serviceModelUpdate(partInfo, userName);
                mesPullingOrderPartInfoRepository.update(partInfo);
            } else {
                MesPcnException.throwMesBusiException("工单拉动组明细【%s】数据为空", mesPullingOrderInfo.getWorkOrderNo());
            }
        } else {
            MesPcnException.throwMesBusiException("工单拉动组明细【%s】数据为空", mesPullingOrderInfo.getWorkOrderNo());
        }
        return partInfo;
    }

    private boolean checkRule(MesPullingOrderPartInfo mesPullingOrderInfo, MesPartPullDetail mesPartPullDetail) {
        String rule = mesPartPullDetail.getRuleValidation();
        if (mesPullingOrderInfo.getScanValidationType() == MesExtEnumUtil.PART_PULL_DETAIL_SCAN_VALIDATION_TYPE.HOMEMADE_VALIDATE.getValue()) {
            List<MesProduceSn> produceSnList = mesProduceSnExtService.getProduceSnList(mesPullingOrderInfo.getOrganizeCode(), mesPullingOrderInfo.getProductSn());
            return mesPartPullDetail.getPullPartNo().equals(produceSnList.get(0).getPartNo());
        } else if (mesPullingOrderInfo.getScanValidationType() == MesExtEnumUtil.PART_PULL_DETAIL_SCAN_VALIDATION_TYPE.COOPERATION_VALIDATE.getValue()) {
            return (Boolean) ((IMesNumberRuleMatchDispatchService) SpringContextsUtil.getBean(MesExtEnumUtil.ASSEMBLY_MATCH_TYPE.MATCH_TYPE_20.getStrategyClass()))
                    .matchNumberRule(mesPullingOrderInfo.getOrganizeCode(), mesPullingOrderInfo.getProductSn(), rule, rule)
                    .get(MesPcnExtConstWords.RESULT);
        } else {
            return true;
        }
    }

    @Override
    public List<MesPullingOrderPartInfo> doMesPullingOrderInfoSend(List<MesPullingOrderPartInfo> infoList, String userName) {
        List<MesPullingOrderPartInfo> result = new ArrayList<>();
        for (MesPullingOrderPartInfo mesPullingOrderPartInfo : infoList) {
            //查询拉动组明细
//            DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(mesPullingOrderPartInfo.getOrganizeCode());
//            DdlPreparedPack.getStringEqualPack(mesPullingOrderPartInfo.getPullingOrderNo(), "pullingOrderNo", partPackBean);
//            DdlPreparedPack.getStringEqualPack(mesPullingOrderPartInfo.getPartNo(), "partNo", partPackBean);
            MesPullingOrderPartInfo partInfo = mesPullingOrderPartInfoRepository.getById(mesPullingOrderPartInfo.getId());
            if (!StringUtil.isEmpty(partInfo)) {
                //修改为已送料
                partInfo.setStatus(MesExtEnumUtil.PULL_ORDER_PART_STATUS.SENTED.getValue());
                partInfo.setColor(MesExtEnumUtil.PART_PULL_DETAIL_COLOR.GREEN.getCode());
                ConvertBean.serviceModelUpdate(partInfo, userName);
                mesPullingOrderPartInfoRepository.update(partInfo);
                //拉动单改为已扫描
                DdlPackBean packBean = DdlPackBean.getDdlPackBean(partInfo.getOrganizeCode());
                DdlPreparedPack.getStringEqualPack(partInfo.getPullingOrderNo(), "pullingOrderNo", packBean);

                MesPullingOrderInfo pullingOrderInfo = mesPullingOrderInfoRepository.getByProperty(packBean);
                if (!StringUtil.isEmpty(pullingOrderInfo)) {
                    pullingOrderInfo.setPullOrderStatus(MesExtEnumUtil.PULL_ORDER_STATUS.JIS_PULL.getValue());
                    ConvertBean.serviceModelUpdate(pullingOrderInfo, userName);
                    mesPullingOrderInfoRepository.update(pullingOrderInfo);
                }
                result.add(partInfo);
            }
        }
        return result;
    }
}
