package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi.printqueue.strategy;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesWorkCenterService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi.printqueue.IPrintQueueStrategyService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.print.IPrintTemplateStrategyService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesProduceSnPrintModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @PROJECT_NAME: i3plus-mes-yfai
 * @DESCRIPTION:
 * @USER: xinwang.yi
 * @DATE: 2024-09-23 11:08
 */
@Slf4j
@Service
public class WorkOrderAssemblyPrintQueueStrategyService implements IPrintQueueStrategyService {

    @Autowired
    private MesPrintQueueRepository printQueueRepository;

    @Autowired
    private MesWorkOrderRepository workOrderRepository;

    @Autowired
    private MesPartProdGroupRepository partProdGroupRepository;

    @Autowired
    private PrintQueueDispatchService printQueueDispatchService;

    @Autowired
    private MesCustomerCarModelRepository customerCarModelRepository;

    @Autowired
    private MesWorkOrderAssemblyRepository workOrderAssemblyRepository;

    @Autowired
    private IMesWorkCenterService workCenterService;

    @Autowired
    private IMesPrintedSnLogRepository printedSnLogRepository;

    @Override
    public List<MesPrintQueue> execute(MesPrintQueue model) {

        // 获取需要打印的打印队列
        List<MesPrintQueue> printQueueList = getMesPrintQueues(model);

        // 无打印直接返回
        if (CollectionUtils.isEmpty(printQueueList)) return null;

        // 获取工单信息
        List<MesWorkOrder> workOrderList = getMesWorkOrders(model, printQueueList);

        // 产线代码集合
        List<String> workCenterCodeList = workOrderList.stream().map(MesWorkOrder::getWorkCenterCode).distinct().collect(Collectors.toList());

        // 根据产线代码节后获取产线信息集合
        List<MesWorkCenter> mesWorkCenters = workCenterService.queryMesWorkCenterList(workCenterCodeList, model.getOrganizeCode());

        // 产线名称集合
        Map<String, String> workCenterNameMap = mesWorkCenters.stream().collect(Collectors.toMap(MesWorkCenter::getWorkCenterCode, MesWorkCenter::getWorkCenterName, (x, y) -> y));

        // 根据工单号对工单分组
        Map<String, MesWorkOrder> workOrderMapByWorkOrderNo = workOrderList.stream().collect(Collectors.toMap(MesWorkOrder::getWorkOrderNo, Function.identity(), (x, y) -> y));

        // 根据零件生产组代码分组
        Map<String, MesPartProdGroup> partProdGroupMap = getMesPartProdGroupMap(model, printQueueList);

        // 客户车型配置名称MAP
        Map<String, MesCustomerCarModel> carModelMapByCode = getMesCustomerCarModelMap(model, workOrderList);

        // 工单装配件
        Map<String, List<MesWorkOrderAssembly>> workOrderAssemblyMapByWorkOrderNo = getWorkOrderAssemblyMapByWorkOrderNo(model, workOrderMapByWorkOrderNo);

        // 展示打印队列列表
        List<MesPrintQueue> showPrintQueueList = new ArrayList<>();

        // 条码打印记录
        List<MesPrintedSnLog> printedSnLogList = new ArrayList<>();

        for (MesPrintQueue printQueue : printQueueList) {

            MesWorkOrder mesWorkOrder = workOrderMapByWorkOrderNo.get(printQueue.getWorkOrderNo());

            if (mesWorkOrder == null) {
                log.info("WorkOrderAssemblyPrintQueueStrategyService --- 工单{}为空 暂不输出打印", printQueue.getWorkOrderNo());
                continue;
            }

            mesWorkOrder.setWorkCenterName(workCenterNameMap.get(printQueue.getWorkCenterCode()));

            printQueue.setWorkOrderTypeName(MesExtEnumUtil.ORDER_TYPE.valueOfDescription(mesWorkOrder.getWorkOrderType()));

            MesPartProdGroup mesPartProdGroup = partProdGroupMap.get(printQueue.getPartProdGroupCode());

            String printTemplate = mesPartProdGroup.getPrintTemplate();

            if (StringUtils.isEmpty(printTemplate)) {
                printQueue.setRemark("打印模板为空");
                printQueue.setSystemSyncStatus(CommonEnumUtil.FALSE);
                ConvertBean.serviceModelUpdate(printQueue, model.getModifyUser());
                continue;
            }

            MesLabelTemplate labelTemplate = printQueueDispatchService.getLabelTemplate(printTemplate, model.getOrganizeCode());

            //根据反射获取策略类--封装打印数据
            String methodCode = labelTemplate.getMethodCode();

            //模板信息丢失抛出异常
            if (StringUtils.isEmpty(methodCode)) {
                printQueue.setRemark(String.format("模板代码[%s]方法类为空,请配置!", labelTemplate.getTemplateCode()));
                printQueue.setSystemSyncStatus(CommonEnumUtil.FALSE);
                ConvertBean.serviceModelUpdate(printQueue, model.getModifyUser());
                continue;
            }

            Map<String, Object> modelMap = new HashMap<>();
            modelMap.put("mesWorkOrder", mesWorkOrder);
            modelMap.put("mesPartProdGroup", mesPartProdGroup);

            MesCustomerCarModel customerCarModel = carModelMapByCode.get(mesWorkOrder.getCarModelCode());
            if (customerCarModel != null) {
                modelMap.put("mesCustomerCarModel", customerCarModel);
            }
            List<MesWorkOrderAssembly> workOrderAssemblyListByWorkOrderNo = workOrderAssemblyMapByWorkOrderNo.get(printQueue.getWorkOrderNo());
            if (!CollectionUtils.isEmpty(workOrderAssemblyListByWorkOrderNo)) {
                modelMap.put("mesWorkOrderAssemblyList", workOrderAssemblyListByWorkOrderNo);
            }


            MesProduceSnPrintModel mesProduceSnPrintModel = new MesProduceSnPrintModel();
            mesProduceSnPrintModel.setMesLabelTemplate(labelTemplate);
            mesProduceSnPrintModel.setSourceData(modelMap);
            mesProduceSnPrintModel.setPrinter(mesPartProdGroup.getPrinter());

            MesProduceSnPrintModel snPrintModel = null;
            try {
                IPrintTemplateStrategyService strategyService = (IPrintTemplateStrategyService) SpringContextsUtil.getBean(methodCode);
                snPrintModel = strategyService.execute(null, mesProduceSnPrintModel, null, null, null ,false);
            } catch (BeansException e) {
                printQueue.setRemark(String.format("模板代码[%s]方法类[%s]执行异常,请配置!", labelTemplate.getTemplateCode(), labelTemplate.getMethodCode()));
                ConvertBean.serviceModelUpdate(printQueue, model.getModifyUser());
                printQueue.setSystemSyncStatus(CommonEnumUtil.FALSE);
                continue;
            }

            printQueue.setPrintStatus(MesExtEnumUtil.PRINT_STATUS.PRINTED.getValue());
            printQueue.setPrintTime(TimeTool.getNowTime(true));
            ConvertBean.serviceModelUpdate(printQueue, model.getModifyUser());
            printQueue.setPrintContent(snPrintModel.getPrintContextList());
            printQueue.setSystemSyncStatus(CommonEnumUtil.FALSE);

            showPrintQueueList.add(printQueue);

            MesPrintedSnLog printedSnLog = new MesPrintedSnLog();
            printedSnLog.setOrganizeCode(printQueue.getOrganizeCode());
            printedSnLog.setWorkOrderNo(printQueue.getWorkOrderNo());
            printedSnLog.setCustPartNo(printQueue.getCustPartNo());
            ConvertBean.serviceModelInitialize(printedSnLog, "JOB");
            printedSnLogList.add(printedSnLog);

        }

        printQueueRepository.saveAll(printQueueList);

        if (!CollectionUtils.isEmpty(printedSnLogList)) printedSnLogRepository.saveAll(printedSnLogList);

        return showPrintQueueList;
    }

    private Map<String, List<MesWorkOrderAssembly>> getWorkOrderAssemblyMapByWorkOrderNo(MesPrintQueue model, Map<String, MesWorkOrder> workOrderMapByWorkOrderNo) {
        List<String> workOrderNoList = workOrderMapByWorkOrderNo.keySet().stream().collect(Collectors.toList());
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(model.getOrganizeCode());
        DdlPreparedPack.getInPackList(workOrderNoList, MesPcnExtConstWords.WORK_ORDER_NO, packBean);
        List<MesWorkOrderAssembly> workOrderAssemblyList = workOrderAssemblyRepository.findByHqlWhere(packBean);
        Map<String, List<MesWorkOrderAssembly>> workOrderAssemblyMapByWorkOrderNo = workOrderAssemblyList.stream().collect(Collectors.groupingBy(MesWorkOrderAssembly::getWorkOrderNo));
        return workOrderAssemblyMapByWorkOrderNo;
    }

    private Map<String, MesCustomerCarModel> getMesCustomerCarModelMap(MesPrintQueue model, List<MesWorkOrder> workOrderList) {
        List<String> carModelCodeList = workOrderList.stream().map(MesWorkOrder::getCarModelCode).distinct().collect(Collectors.toList());
        // 客户车型配置
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(model.getOrganizeCode());
        DdlPreparedPack.getInPackList(carModelCodeList, MesPcnExtConstWords.CAR_MODEL_CODE, packBean);
        List<MesCustomerCarModel> customerCarModelList = customerCarModelRepository.findByHqlWhere(packBean);

        Map<String, MesCustomerCarModel> carModelMapByCode = customerCarModelList.stream().collect(Collectors.toMap(MesCustomerCarModel::getCarModelCode, Function.identity(), (x, y) -> y));
        return carModelMapByCode;
    }

    private Map<String, MesPartProdGroup> getMesPartProdGroupMap(MesPrintQueue model, List<MesPrintQueue> printQueueList) {
        // 生产零件号
        List<String> partProdGroupCodeList =  printQueueList.stream().map(MesPrintQueue::getPartProdGroupCode).distinct().collect(Collectors.toList());
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(model.getOrganizeCode());
        //DdlPreparedPack.getInPackList(workCenterCodeList, MesPcnExtConstWords.WORK_CENTER_CODE, packBean);
        DdlPreparedPack.getInPackList(partProdGroupCodeList, MesPcnExtConstWords.PART_PROD_GROUP_CODE, packBean);
        List<MesPartProdGroup> partProdGroupList = partProdGroupRepository.findByHqlWhere(packBean);
        return partProdGroupList.stream().collect(Collectors.toMap(MesPartProdGroup::getPartProdGroupCode, Function.identity(), (x,y) -> y));
    }

    private List<MesWorkOrder> getMesWorkOrders(MesPrintQueue model, List<MesPrintQueue> printQueueList) {
        List<String> workOrderNoList = printQueueList.stream().map(MesPrintQueue::getWorkOrderNo).distinct().collect(Collectors.toList());
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(model.getOrganizeCode());
        DdlPreparedPack.getInPackList(workOrderNoList, MesPcnExtConstWords.WORK_ORDER_NO, packBean);
        return workOrderRepository.findByHqlWhere(packBean);
    }

    private List<MesPrintQueue> getMesPrintQueues(MesPrintQueue model) {
        Pager pager = new Pager();
        pager.setStartRow(1);
        pager.setCurrentPage(1);
        pager.setPageSize(30);
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(model.getOrganizeCode());
        DdlPreparedPack.getInPackList(Arrays.asList(model.getWorkCenterCodeQuery().split(MesPcnExtConstWords.COMMA)), MesPcnExtConstWords.WORK_CENTER_CODE, packBean);
        DdlPreparedPack.getInPackList(Arrays.asList(model.getPartProdGroupCodeQuery().split(MesPcnExtConstWords.COMMA)), MesPcnExtConstWords.PART_PROD_GROUP_CODE, packBean);
        DdlPreparedPack.getIsNull(MesPcnExtConstWords.REMARK, packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.PRINT_QUEUE_TYPE.WORK_ORDER_ASSEMBLY.getValue(), MesPcnExtConstWords.PRINT_QUEUE_TYPE, packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.PRINT_STATUS.UNPRINT.getValue(), MesPcnExtConstWords.PRINT_STATUS, packBean);
        return printQueueRepository.findByHqlWherePage(packBean, pager);
    }

}
