package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi.shippingscan.strategy;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesConfigService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProduceSnExtService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesWorkOrderExtService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.aspect.MonitorLog;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi.shippingscan.IMesShippingScanStrategyService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.ext.mes.pcn.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesSortShippingCheckModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesPartShippingGroup;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrder;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesShippingOrderManagement;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesShippingOrderManagementDetail;
import cn.estsh.i3plus.pojo.mes.repository.MesCustomerPartRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesProduceSnRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesShippingOrderManagementDetailRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesShippingOrderManagementRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import groovy.util.logging.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @Description : 扫描条码和目视单
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/9/25 10:54
 * @Modify:
 **/
@Service
@Slf4j
public class MesShippingScanSnAndOrderStrategyServiceImpl implements IMesShippingScanStrategyService {
    @Autowired
    private MesShippingOrderManagementRepository shippingOrderManagementRepository;

    @Autowired
    private MesShippingOrderManagementDetailRepository shippingOrderManagementDetailRepository;

    @Autowired
    private MesProduceSnRepository produceSnRepository;

    @Autowired
    private IMesProduceSnExtService produceSnExtService;

    @Autowired
    private MesCustomerPartRepository mesCustomerPartRepository;

    @Autowired
    private IMesConfigService mesConfigService;

    @Autowired
    private IMesWorkOrderExtService mesWorkOrderExtService;

    @Override
    public MesSortShippingCheckModel doScan(MesSortShippingCheckModel model, MesShippingOrderManagement orderManagement, MesPartShippingGroup shippingGroup) {
        if (!Objects.isNull(model.getLastShippingDetailId())) {
            //校验目视单是否已经扫描
            if (model.getDetailList().stream().anyMatch(t -> !StringUtils.isEmpty(t.getVisualOrderNo()) && t.getVisualOrderNo().equals(model.getSn()))) {
                throw new ImppBusiException(String.format("【%s】此目视单已经扫描过，请检查数据！", model.getSn()));
            }
            //更新单据信息
            model.getDetailList().stream().filter(t -> t.getId().equals(model.getLastShippingDetailId())).forEach(detail -> {
                detail.setVisualOrderNo(model.getSn());
                updateOrderManagementDetails(orderManagement, detail, getWorkOrder(model, detail), model.getUserInfo());
            });
            model.setLastShippingDetailId(null);
        } else {
            //校验条码
            MesProduceSn produceSn = checkSn(model);
            boolean scanFlg = false;
            for (MesShippingOrderManagementDetail detail : model.getDetailList()) {
                if (detail.getPartNo().equals(produceSn.getPartNo()) && StringUtils.isEmpty(detail.getBarcode()) && !Objects.equals(detail.getIsScanFlg(), MesCommonConstant.TRUE_INTEGER)) {
                    //校验发运明细
                    checkShippingDetails(model, orderManagement, shippingGroup, produceSn, detail);
                    detail.setBarcode(model.getSn());
                    //校验通过的明细ID
                    model.setLastShippingDetailId(detail.getId());
                    scanFlg = true;
                    break;
                }
            }
            if (!scanFlg) {
                throw new ImppBusiException(String.format("【%s】此条码对应的【%s】物料号与发运单中物料号不匹配，请检查数据！", model.getSn(), produceSn.getPartNo()));
            }
        }

        return model;
    }


    private MesWorkOrder getWorkOrder(MesSortShippingCheckModel model, MesShippingOrderManagementDetail detail) {
        MesWorkOrder workOrder = mesWorkOrderExtService.getWorkOrder(model.getOrganizeCode(), model.getSn());
        if (Objects.isNull(workOrder)) {
            throw new ImppBusiException(String.format("目视单【%s】不存在，请检查数据！", model.getSn()));
        }
        //校验目视单状态
        if (MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue() != workOrder.getWorkOrderStatus() || MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue() != workOrder.getQcStatus()) {
            throw new ImppBusiException(String.format("目视单【%s】状态为【%s】质量状态为【%s】，不允许发运！", model.getSn(), MesExtEnumUtil.ORDER_STATUS.valueOfDescription(workOrder.getWorkOrderStatus()), MesExtEnumUtil.PRODUCE_QC_STATUS.valueOfDescription(workOrder.getQcStatus())));
        }
        //校验条码是否匹配
        if (!Objects.equals(detail.getBarcode(), workOrder.getSn())) {
            throw new ImppBusiException(String.format("目视单【%s】条码【%s】与扫描条码【%s】，不匹配！", model.getSn(), workOrder.getSn(), detail.getBarcode()));
        }
        //校验零件是否匹配
        if (!Objects.equals(detail.getPartNo(), workOrder.getPartNo())) {
            throw new ImppBusiException(String.format("目视单【%s】零件【%s】与发运单零件【%s】，不匹配！", model.getSn(), workOrder.getPartNo(), detail.getPartNo()));
        }
        return workOrder;
    }

    private void checkShippingDetails(MesSortShippingCheckModel model, MesShippingOrderManagement orderManagement, MesPartShippingGroup shippingGroup, MesProduceSn produceSn, MesShippingOrderManagementDetail detail) {
        //校验发运单明细顺序
        if (!Objects.isNull(shippingGroup) && !StringUtils.isEmpty(shippingGroup.getIsEnableShippingFailSafe()) && CommonEnumUtil.VALID == shippingGroup.getIsEnableShippingFailSafe()) {
            Optional<MesShippingOrderManagementDetail> first = model.getDetailList().stream().filter(k -> k.getCustInfoSeq().compareTo(detail.getCustInfoSeq()) < 0 && !Objects.equals(k.getIsScanFlg(), MesCommonConstant.TRUE_INTEGER)).findFirst();
            if (first.isPresent()) {
                throw new ImppBusiException(String.format("发运单【%s】启用明细顺序防错，序号【%s】前还有未扫描的发运单明细，请检查数据！", orderManagement.getShippingCode(), detail.getCustInfoSeq()));
            }
        }
        //判断是否需要排序校验 默认排序则需要校验
        if (!StringUtils.isEmpty(orderManagement.getScanSeqWay()) && orderManagement.getScanSeqWay() == MesExtEnumUtil.SHIPPING_GROUP_SCAN_CONFIRM_SEQ_MODE.DEFAULT_SORT.getValue()) {
            //获取明细中最小的
            Optional<String> minPartNo = model.getDetailList().stream().filter(k -> StringUtils.isEmpty(k.getBarcode()))
                    .min(Comparator.comparingLong(MesShippingOrderManagementDetail::getCustInfoSeq)).map(MesShippingOrderManagementDetail::getPartNo);
            //若扫描的不是最小顺序 则报错
            if (minPartNo.isPresent() && !minPartNo.get().equals(produceSn.getPartNo())) {
                throw new ImppBusiException(String.format("【%s】此条码扫描顺序错误，请检查数据！", model.getSn()));
            }
        }
        //校验客户零件信息是否存在
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(model.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(detail.getPartNo(), "erpPartNo", packBean);
        DdlPreparedPack.getStringEqualPack(detail.getCustPartNo(), "custPartNo", packBean);
        if (!mesCustomerPartRepository.isExitByHql(packBean)) {
            throw new ImppBusiException(String.format("零件号【%s】客户零件号【%s】关系不存在，请检查数据！", detail.getPartNo(), detail.getCustPartNo()));
        }
    }

    private MesProduceSn checkSn(MesSortShippingCheckModel model) {
        //校验条码是否重复扫描
        Optional<MesShippingOrderManagementDetail> optional = model.getDetailList().stream().filter(k -> !StringUtils.isEmpty(k.getBarcode()) && k.getBarcode().equals(model.getSn())).findFirst();
        if (optional.isPresent()) {
            throw new ImppBusiException(String.format("【%s】此条码已经扫描过，请检查数据！", model.getSn()));
        }
        MesProduceSn produceSn = getMesProduceSn(model.getOrganizeCode(), model.getSn());
        if (!StringUtils.isEmpty(produceSn)) {
            //校验状态
            checkStatus(model, produceSn);
        } else {
            //外部条码解析
            MesShippingOrderManagementDetail mesShippingOrderManagementDetail = getMesShippingOrderManagementDetail(model);
            //封装信息
            produceSn = new MesProduceSn();
            produceSn.setSerialNumber(model.getSn());
            produceSn.setProductSn(model.getSn());
            produceSn.setPartNo(mesShippingOrderManagementDetail.getPartNo());
        }
        return produceSn;
    }

    private MesProduceSn getMesProduceSn(String organizeCode, String productSn) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(productSn, "custSn", packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
        return produceSnRepository.getByProperty(packBean);
    }

    private void checkStatus(MesSortShippingCheckModel model, MesProduceSn produceSn) {
        //校验产品条码状态 必须是状态为已下线条码
        if (produceSn.getSnStatus() != MesExtEnumUtil.PRODUCE_SN_STATUS.OFFLINE.getValue() && produceSn.getSnStatus() != MesExtEnumUtil.PRODUCE_SN_STATUS.CREATE.getValue()) {
            throw new ImppBusiException(String.format("【%s】此条码状态为【%s】,请选择状态为【%s】或【%s】状态的条码", model.getSn(), MesExtEnumUtil.PRODUCE_SN_STATUS.valueOfDescription(produceSn.getSnStatus()),
                    MesExtEnumUtil.PRODUCE_SN_STATUS.OFFLINE.getDescription(), MesExtEnumUtil.PRODUCE_SN_STATUS.CREATE.getDescription()));
        }
        //校验产品质量状态 必须是状态为合格条码
        if (produceSn.getQcStatus() != MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue()) {
            throw new ImppBusiException(String.format("【%s】此条码质量状态为【%s】,请选择【%s】状态的条码", model.getSn(), MesExtEnumUtil.PRODUCE_QC_STATUS.valueOfDescription(produceSn.getQcStatus()),
                    MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getDescription()));
        }
    }

    private MesShippingOrderManagementDetail getMesShippingOrderManagementDetail(MesSortShippingCheckModel model) {
        //外部解析条码长度卡控
        String cfgValue = mesConfigService.getCfgValue(model.getOrganizeCode(), MesPcnExtConstWords.MAX_SHIPPING_BARCODE_LENGTH);
        if (Integer.parseInt(cfgValue) < model.getSn().length())
            MesPcnException.throwMesBusiException("【%s】此条码长度超过【%s】位，请检查数据！", model.getSn(), cfgValue);
        List<MesShippingOrderManagementDetail> detailList = model.getDetailList().stream().filter(k -> !StringUtils.isEmpty(k.getCustPartNo()) && model.getSn().startsWith(k.getCustPartNo()) && StringUtils.isEmpty(k.getBarcode()) && !Objects.equals(k.getIsScanFlg(), MesCommonConstant.TRUE_INTEGER)).collect(Collectors.toList());
        MesShippingOrderManagementDetail mesShippingOrderManagementDetail = null;
        for (MesShippingOrderManagementDetail managementDetail : detailList) {
            DdlPackBean packBean = DdlPackBean.getDdlPackBean(model.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(managementDetail.getPartNo(), "erpPartNo", packBean);
            DdlPreparedPack.getStringEqualPack(managementDetail.getCustPartNo(), "custPartNo", packBean);
            if (mesCustomerPartRepository.isExitByHql(packBean)) {
                mesShippingOrderManagementDetail = managementDetail;
                break;
            }
        }
        if (Objects.isNull(mesShippingOrderManagementDetail)) {
            throw new ImppBusiException(String.format("【%s】此条码未匹配到数据，请检查数据！", model.getSn()));
        }
        return mesShippingOrderManagementDetail;
    }

    private void updateOrderManagementDetails(MesShippingOrderManagement orderManagement, MesShippingOrderManagementDetail detail, MesWorkOrder workOrder, String userInfo) {
        if (StringUtils.isEmpty(orderManagement.getCheckSeqCode())) {
            detail.setIsScanFlg(MesCommonConstant.TRUE_INTEGER);
            detail.setStatus(MesExtEnumUtil.SHIPPING_ORDER_DETAIL_SHIPPING_STATUS.SHIPPINGED.getValue());
            detail.setRemark("扫描完成");
            //根据是否启动记忆判断 若启用记忆，则根据保存数据库
            if (!StringUtils.isEmpty(orderManagement.getIsEnableMemory()) && Objects.equals(orderManagement.getIsEnableMemory(), MesCommonConstant.TRUE_INTEGER)) {
                //若发运单状态为创建或发布 则写入开始扫描时间
                if (!StringUtils.isEmpty(orderManagement.getStatus()) && (orderManagement.getStatus() == MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue() ||
                        orderManagement.getStatus() == MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue())) {
                    orderManagement.setStartScanTime(TimeTool.getNowTime(true));
                }

                orderManagement.setStatus(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPING.getValue());
                orderManagement.setSystemSyncStatus(CommonEnumUtil.FALSE);
                ConvertBean.serviceModelUpdate(orderManagement, userInfo);
                update(orderManagement);

                detail.setActualQty(1);
                detail.setSystemSyncStatus(CommonEnumUtil.FALSE);
                ConvertBean.serviceModelUpdate(detail, userInfo);
                updateDetail(detail);
                MesProduceSn produceSn = getMesProduceSn(orderManagement.getOrganizeCode(), detail.getBarcode());
                //内部条码更新条码状态
                if (!Objects.isNull(produceSn)) {
                    produceSn.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.SHIPPED.getValue());
                    produceSn.setSystemSyncStatus(CommonEnumUtil.FALSE);
                    ConvertBean.serviceModelUpdate(produceSn, userInfo);
                    produceSnExtService.update(produceSn);
                }
                //更新工单状态为发运
                workOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.SHIPPING.getValue());
                ConvertBean.serviceModelUpdate(workOrder, userInfo);
                mesWorkOrderExtService.update(workOrder);
            }
        }
    }


    @MonitorLog
    public void update(MesShippingOrderManagement item) {
        shippingOrderManagementRepository.update(item);
    }

    @MonitorLog
    public void updateDetail(MesShippingOrderManagementDetail item) {
        shippingOrderManagementDetailRepository.update(item);
    }

}
