package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.numberrule;

import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.api.iservice.busi.INumberRulePackAttributeStrategyService;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesCustomerPart;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.MesCustomerPartRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.Map;
import java.util.Objects;
import java.util.StringJoiner;

/**
 * @Description : 门板及cnsl总成条码
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/9/27 13:23
 * @Modify:
 **/
@Component
public class MesDPBarcodeWhNumberRuleStrategyService implements INumberRulePackAttributeStrategyService {

    @Autowired
    private MesCustomerPartRepository customerPartRepository;

    @Override
    public GenSerialNoModel execute(GenSerialNoModel genSerialNoModel) {
        //获取客户零件号
        Map<String, Object> dataMap = genSerialNoModel.getDataMap();
        MesCustomerPart customerPart = (!CollectionUtils.isEmpty(dataMap) && dataMap.containsKey(MesCustomerPart.class.getSimpleName())) ?
                (MesCustomerPart) dataMap.get(MesCustomerPart.class.getSimpleName()) : getMesCustomerPart(genSerialNoModel.getOrganizeCode(), genSerialNoModel.getPartNo());
        if (StringUtils.isEmpty(genSerialNoModel.getShiftCode()) || Objects.isNull(customerPart)) {
            MesPcnException.throwMesBusiException("班次代码或客户零件号未维护");
        }
        genSerialNoModel.setCustPartNo(customerPart.getCustPartNo());
        genSerialNoModel.setDynamicRule(new StringJoiner(MesPcnExtConstWords.COMMA).add(customerPart.getCustPartNo().substring(customerPart.getCustPartNo().length() - 4)).add(genSerialNoModel.getShiftCode()).toString());
        Date date = new Date();
        //年月日缩写
        genSerialNoModel.setYear(getYearShort(date));
        genSerialNoModel.setMonth(getMonthShort(date));
        genSerialNoModel.setDay(getDayShort(date));
        return genSerialNoModel;
    }

    private String getYearShort(Date date) {
        return MesExtEnumUtil.YEAR_SHORT.valueOfDescription(Integer.parseInt(TimeTool.getYear(date)));
    }

    private String getMonthShort(Date date) {
        return MesExtEnumUtil.MONTH_SHORT.valueOfDescription(Integer.parseInt(TimeTool.getMonth(date)));
    }

    private String getDayShort(Date date) {
        return TimeTool.getDay(date);
    }

    private MesCustomerPart getMesCustomerPart(String orgaizeCode, String partNo) {
        if (StringUtils.isEmpty(orgaizeCode) || StringUtils.isEmpty(partNo)) return null;
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(orgaizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, MesPcnExtConstWords.ERP_PART_NO, ddlPackBean);
        MesCustomerPart customerPart = customerPartRepository.getByProperty(ddlPackBean);
        if (Objects.isNull(customerPart)) {
            MesPcnException.throwMesBusiException("物料【%s】客户零件关系未维护", partNo);
        }
        return customerPart;
    }
}
