package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.station;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesProdShiftRecordService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionCustomContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionDispatchContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesWorkOrderExtService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProdShiftContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionPartContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionProcessContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseModuleService;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.IStepService;
import cn.estsh.i3plus.mes.pcn.util.StationKvBeanUtil;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.AttrBean;
import cn.estsh.i3plus.pojo.mes.model.StationKvBean;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.repository.MesFileRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.i3plus.pojo.mes.util.PojoAttrUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import com.alibaba.fastjson.JSONObject;
import com.google.common.base.Objects;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.StringJoiner;
import java.util.function.BinaryOperator;

/**
 * @Description : 展示组件：非排序生产
 * @Author :wangjie
 **/
@Slf4j
@Service("mesProductionNoSortModuleService")
public class MesProductionNoSortModuleService extends BaseModuleService {

    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;

    @Autowired
    private IMesProductionDispatchContextStepService productionDispatchContextStepService;

    @Autowired
    private IMesProductionCustomContextStepService productionCustomContextStepService;

    @Autowired
    private IMesProdShiftRecordService mesProdShiftRecordService;

    @Autowired
    private IMesWorkOrderExtService workOrderExtService;

    @Autowired
    private MesFileRepository mesFileRepository;

    @Override
    public void init(StationRequestBean reqBean) {
        // 获取工单信息
        List<List<StationKvBean>> moduleContentContext = productionDispatchContextStepService.getModuleContentContext(reqBean);

        //获取上下文信息
        MesProductionProcessContext productionProcessContext = productionProcessContextStepService.getEquipmentVariableList(reqBean, MesExtEnumUtil.EQUIP_VARIABLE_TYPE.PRODUCTION.getValue());
        MesWorkCell mesWorkCell = productionProcessContext.getWorkCell();

        //获取班次信息，先查询缓存，缓存没有在查询数据库
        List<StationKvBean> prodShiftDataContext = getShiftRecordStationKvBeans(reqBean);

        MesProdShiftContext mesProdShiftKvBean = productionCustomContextStepService.getMesProdShiftKvBean(reqBean.getOrganizeCode(), reqBean.getWorkCenterCode());

        if (CollectionUtils.isEmpty(prodShiftDataContext)) {
            this.sendMessage(reqBean, new StationResultBean().writeDbLog(), "请先开班",  MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.TEXT);
            MesPcnException.throwBusiException("请先开班");
        }
        // 发送班次班组和工单
        StationResultBean resultBean = getStationResultBean(reqBean, moduleContentContext, prodShiftDataContext);
        this.sendMessage(reqBean, resultBean);

         // 整合班次加工数量
        List<StationKvBean> productionStatisticsContext = productionCustomContextStepService.getProductionStatisticsContext(reqBean);
        if (CollectionUtils.isEmpty(productionStatisticsContext)) {
            productionStatisticsContext = StationKvBeanUtil.addStationKvBeanList(new ArrayList<>(), new StationKvBean(new StringJoiner(MesPcnExtConstWords.AND).add(mesProdShiftKvBean.getShiftGroup()).add(mesProdShiftKvBean.getShiftCode()).toString(), "加工数", MesPcnExtConstWords.ZERO_STR),
                    new StationKvBean(MesPcnExtConstWords.COLOR, "颜色", MesExtEnumUtil.COLOR.BLACK.getValue()));
        }
        StationResultBean shiftCountBean = getStationResultBean(reqBean, productionStatisticsContext,mesWorkCell, mesProdShiftKvBean);
        this.sendMessage(reqBean, shiftCountBean);

        //验证工位是否锁定
        reqBean.getDataMap().put(MesPcnEnumUtil.ACTOR_RECEIVE_STRATEGY.WS_CMD_INIT_MODULE.getCode(), CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValueStr());
        ((IStepService) SpringContextsUtil.getBean("mesCountDownShowStepService")).executeInState(reqBean);

        //设备BYPASS监控
        ((IStepService) SpringContextsUtil.getBean("mesEquipByPassReadStepService")).executeInState(reqBean);

    }

    private List<StationKvBean> getShiftRecordStationKvBeans(StationRequestBean reqBean) {
        // 获取班次信息 redis
        List<StationKvBean> prodShiftDataContext = productionCustomContextStepService.getProdShiftDataContext(reqBean.getOrganizeCode(), reqBean.getWorkCenterCode());
        if (CollectionUtils.isEmpty(prodShiftDataContext)) {
            MesProdShiftRecord shiftRecord = new MesProdShiftRecord();
            shiftRecord.setWorkCenterCode(reqBean.getWorkCenterCode());
            shiftRecord.setWorkStatus(MesExtEnumUtil.WORK_STATUS.START.getValue());
            // 查询数据库
            MesProdShiftRecord prodShiftRecord = mesProdShiftRecordService.queryMesProdShiftRecord(shiftRecord);
            if (prodShiftRecord != null) {
                prodShiftDataContext = getProdShiftData(prodShiftRecord);
                productionCustomContextStepService.dispatchProdShiftDataContext(reqBean.getOrganizeCode(), reqBean.getWorkCenterCode(),prodShiftDataContext);
            }
        }
        return prodShiftDataContext;
    }

    private StationResultBean getStationResultBean(StationRequestBean reqBean, List<List<StationKvBean>> moduleContentContext, List<StationKvBean> prodShiftDataContext) {
        StationResultBean resultBean = new StationResultBean();
        resultBean.setBusiType(MesPcnEnumUtil.STATION_BUSI_TYPE.MODULE_CONTENT.getValue());
        resultBean.setDataType(MesPcnEnumUtil.STATION_DATA_TYPE.CUSTOM.getValue());
        resultBean.setCustomPageName(MesPcnExtConstWords.CUSTOM_PAGE_NAME_DEFAULT);
        resultBean.setSpecialDisplayData(getStepColIndent(reqBean));


        resultBean.setResultObj(prodShiftDataContext);
        resultBean.setResultList(moduleContentContext);
        return resultBean;
    }

    private StationResultBean getStationResultBean(StationRequestBean reqBean, List<StationKvBean> productionStatisticsContext, MesWorkCell mesWorkCell, MesProdShiftContext mesProdShiftKvBean) {
        StationResultBean resultBean = new StationResultBean();
        resultBean.setBusiType(MesPcnEnumUtil.STATION_BUSI_TYPE.MODULE_CUSTOM_CONTENT.getValue());
        String dataType = MesPcnEnumUtil.STATION_DATA_TYPE.ECHART.getValue();
        JSONObject reqJson = new JSONObject();
        String url = "";

        List<MesProductionPartContext> productionPartContextList = productionDispatchContextStepService.getProductionPartContext(reqBean);

        List<List<StationKvBean>> orderQtyKvBeans = new ArrayList<>();
        if (!StringUtils.isEmpty(mesWorkCell.getFileId())) {
            DdlPackBean packBean = DdlPackBean.getDdlPackBean();

            DdlPreparedPack.getNumEqualPack(Long.parseLong(mesWorkCell.getFileId()), MesPcnExtConstWords.ID, packBean);
            MesFile mesFile = mesFileRepository.getByProperty(packBean);
            if (mesFile != null) {
                url = mesFile.getFileUrl();
            }
        }
        if (Objects.equal(MesExtEnumUtil.WORK_FILE_TYPE.HISTOGRAM.getValue(), mesWorkCell.getIsShowMsg())) {
            dataType = MesPcnEnumUtil.STATION_DATA_TYPE.ECHART.getValue();
            List<MesWorkOrder> workOrderList = workOrderExtService.getWorkOrderListByShiftCode(reqBean.getOrganizeCode(), mesWorkCell.getWorkCenterCode(), mesProdShiftKvBean.getShiftCode());
            if (CollectionUtils.isEmpty(workOrderList)) {
                StationKvBeanUtil.addStationKvBeanList(orderQtyKvBeans, new ArrayList<>(),
                        new StationKvBean(MesPcnExtConstWords.QTY, "工单计划数", MesPcnExtConstWords.ZERO_STR),
                        new StationKvBean(MesPcnExtConstWords.COMPLATED_QTY, "工单完成数", MesPcnExtConstWords.ZERO_STR));
            } else {
                Double qty = workOrderList.stream().map(MesWorkOrder::getQty).reduce((a, b) -> a + b).get();
                Double complateQty = workOrderList.stream().map(MesWorkOrder::getCompleteQty).reduce((a, b) -> a + b).get();

                StationKvBeanUtil.addStationKvBeanList(orderQtyKvBeans, new ArrayList<>(),
                        new StationKvBean(MesPcnExtConstWords.QTY, "工单计划数", qty.toString()),
                        new StationKvBean(MesPcnExtConstWords.COMPLATED_QTY, "工单完成数", complateQty.toString()));
            }
            if (!CollectionUtils.isEmpty(productionPartContextList)) {


            }
            resultBean.setResultList(orderQtyKvBeans);
        } else if (Objects.equal(MesExtEnumUtil.WORK_FILE_TYPE.PROD_VIDEO.getValue(), mesWorkCell.getIsShowMsg())) {
            dataType = MesPcnEnumUtil.STATION_DATA_TYPE.VIDEO.getValue();
            reqJson.put(MesPcnExtConstWords.URL, url);
            resultBean.setResultObj(reqJson);
        }  else if (Objects.equal(MesExtEnumUtil.WORK_FILE_TYPE.WORK_CELL_PICTURE.getValue(), mesWorkCell.getIsShowMsg())) {
            dataType = MesPcnEnumUtil.STATION_DATA_TYPE.IMAGE.getValue();
            reqJson.put(MesPcnExtConstWords.URL, url);
            resultBean.setResultObj(reqJson);
        }

        if (Objects.equal(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue(),mesWorkCell.getIsCountFinish())) {
            dataType = MesPcnEnumUtil.STATION_DATA_TYPE.TEXT.getValue();
            resultBean.setResultList(productionStatisticsContext);
        }
        resultBean.setDataType(dataType);
        resultBean.setCustomPageName(MesPcnExtConstWords.CUSTOM_PAGE_NAME_WORK_CELL);



        return resultBean;
    }

    @Override
    public boolean execStateModule(StationRequestBean reqBean, List<MesStateMachineStatus> states, Map<String, String> wcpcMap) {
        init(reqBean);
        return true;
    }

    //封装展示组件班组班次内容
    private List<StationKvBean> getProdShiftData(MesProdShiftRecord record) {

        return  StationKvBeanUtil.addStationKvBeanList(new ArrayList<>(),
                new StationKvBean(MesPcnExtConstWords.SHIFT_GROUP, "班组", record.getShiftGroup()),
                new StationKvBean(MesPcnExtConstWords.SHIFT_GROUP_NAME, "班组名称", record.getShiftGroupName()),
                new StationKvBean(MesPcnExtConstWords.SHIFT_CODE, "班次", record.getShiftCode()),
                new StationKvBean(MesPcnExtConstWords.SHIFT_NAME, "班次名称", record.getShiftName()));
    }
}
