package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.station;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesProdShiftRecordService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionCustomContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionDispatchContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProdShiftContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionProcessContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesQueueOrderModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseModuleService;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.IStepService;
import cn.estsh.i3plus.mes.pcn.util.PojoAttrUtil;
import cn.estsh.i3plus.mes.pcn.util.StationKvBeanUtil;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.AttrBean;
import cn.estsh.i3plus.pojo.mes.model.StationKvBean;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.repository.MesQueueOrderRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description : 展示组件：排序生产
 * @Author :wangjie
 **/
@Slf4j
@Service("mesProductionSortModuleService")
public class MesProductionSortModuleService extends BaseModuleService {

    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;

    @Autowired
    private IMesProductionDispatchContextStepService productionDispatchContextStepService;

    @Autowired
    private IMesProductionCustomContextStepService productionCustomContextStepService;

    @Autowired
    private IMesProdShiftRecordService mesProdShiftRecordService;

    @Autowired
    private MesQueueOrderRepository queueOrderRepository;

    @Override
    public void init(StationRequestBean reqBean) {
        // 获取工单信息
        List<List<StationKvBean>> moduleContentContext = productionDispatchContextStepService.getModuleContentContext(reqBean);

        //获取上下文信息
        MesProductionProcessContext productionProcessContext = productionProcessContextStepService.getEquipmentVariableList(reqBean, MesExtEnumUtil.EQUIP_VARIABLE_TYPE.PRODUCTION.getValue());
        MesWorkCell mesWorkCell = productionProcessContext.getWorkCell();

        //获取班次信息，先查询缓存，缓存没有在查询数据库
        List<StationKvBean> prodShiftDataContext = getShiftRecordStationKvBeans(reqBean);

        MesProdShiftContext mesProdShiftKvBean = productionCustomContextStepService.getMesProdShiftKvBean(reqBean.getOrganizeCode(), reqBean.getWorkCenterCode());

        if (CollectionUtils.isEmpty(prodShiftDataContext)) {
            this.sendMessage(reqBean, new StationResultBean().writeDbLog(), "请先开班",  MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.TEXT);
            MesPcnException.throwBusiException("请先开班");
        }
        // 发送班次班组和工单
        StationResultBean resultBean = getStationResultBean(reqBean, moduleContentContext, prodShiftDataContext);
        this.sendMessage(reqBean, resultBean);

        // 获取当前工位最新修改的完成状态的队列单据数据 并根据其零件生产组代码按生产序号倒序前50条
        resultBean = getStationResultBean(queryQueueOrderTopFiftyProduceSeq(getNewQueueOrder(reqBean)));
        this.sendMessage(reqBean, resultBean);

        //验证工位是否锁定
        reqBean.getDataMap().put(MesPcnEnumUtil.ACTOR_RECEIVE_STRATEGY.WS_CMD_INIT_MODULE.getCode(), CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValueStr());
        ((IStepService) SpringContextsUtil.getBean("mesCountDownShowStepService")).executeInState(reqBean);

        //设备BYPASS监控
        //((IStepService) SpringContextsUtil.getBean("mesEquipByPassReadStepService")).executeInState(reqBean);

    }

    private StationResultBean getStationResultBean(List<MesQueueOrderModel> mesQueueOrderModelList) {
        return new StationResultBean().busiType(MesPcnEnumUtil.STATION_BUSI_TYPE.MODULE_CUSTOM_CONTENT.getValue())
                .customPageName(MesPcnExtConstWords.CUSTOM_PAGE_NAME_WORK_CELL)
                .dataType(MesPcnEnumUtil.STATION_DATA_TYPE.TABLE.getValue())
                .dataAttrList(dataAttrList()).resultList(mesQueueOrderModelList);
    }

    //队列工单清单列表标题
    public List<AttrBean> dataAttrList() {
        List<AttrBean> attrBeanList = new ArrayList<>();
        PojoAttrUtil.loadPojoAttrs(attrBeanList, MesPcnExtConstWords.PRODUCE_SEQ, "工单序号");
        PojoAttrUtil.loadPojoAttrs(attrBeanList, MesPcnExtConstWords.CAR_MODEL_CODE, "车型配置");
        PojoAttrUtil.loadPojoAttrs(attrBeanList, MesPcnExtConstWords.CUST_ORDER_NO, "客户订单号");
        PojoAttrUtil.loadPojoAttrs(attrBeanList, MesPcnExtConstWords.PART_NO, "零件号");
        return attrBeanList;
    }

    private List<MesQueueOrderModel> queryQueueOrderTopFiftyProduceSeq(MesQueueOrder queueOrder) {

        if (queueOrder == null) return new ArrayList<>();

        Pager pager = new Pager();
        pager.setStartRow(1);
        pager.setCurrentPage(1);
        pager.setPageSize(50);
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(queueOrder.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(queueOrder.getWorkCenterCode(), MesPcnExtConstWords.WORK_CENTER_CODE, packBean);
        DdlPreparedPack.getStringEqualPack(queueOrder.getWorkCellCode(), MesPcnExtConstWords.WORK_CELL_CODE, packBean);
        DdlPreparedPack.getStringEqualPack(queueOrder.getPartProdGroupCode(), MesPcnExtConstWords.PART_PROD_GROUP_CODE, packBean);
        packBean.setOrderByStr(" order by produceSeq desc ");
        List<MesQueueOrder> queueOrderList = queueOrderRepository.findByHqlWherePage(packBean, pager);

        List<MesQueueOrderModel> queueOrderModels = new ArrayList<>();
        if (!CollectionUtils.isEmpty(queueOrderList)) {

            boolean isComplete = false;
            for (MesQueueOrder mesQueueOrder : queueOrderList) {
                MesQueueOrderModel queueOrderModel = new MesQueueOrderModel();
                BeanUtils.copyProperties(mesQueueOrder, queueOrderModel);
                String color = MesExtEnumUtil.COLOR.GREEN.getValue();
                if (queueOrderModel.getStatus() == MesExtEnumUtil.QUEUE_ORDER_STATUS.FINISH.getValue()) isComplete = true;
                else if (!isComplete) color = MesExtEnumUtil.COLOR.WHITE.getValue();
                else if (isComplete && (queueOrderModel.getStatus() == MesExtEnumUtil.QUEUE_ORDER_STATUS.CREATE.getValue())) color = MesExtEnumUtil.COLOR.YELLOW.getValue();
                queueOrderModel.setColor(color);
                queueOrderModels.add(queueOrderModel);
            }
        }

        return queueOrderModels;
    }

    private MesQueueOrder getNewQueueOrder(StationRequestBean reqBean) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(reqBean.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(reqBean.getWorkCenterCode(), MesPcnExtConstWords.WORK_CENTER_CODE, packBean);
        DdlPreparedPack.getStringEqualPack(reqBean.getWorkCellCode(), MesPcnExtConstWords.WORK_CELL_CODE, packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.QUEUE_ORDER_STATUS.FINISH.getValue(), MesPcnExtConstWords.STATUS, packBean);
        packBean.setOrderByStr(" order by modifyDatetime desc ");
        return queueOrderRepository.getByProperty(packBean);
    }

    private List<StationKvBean> getShiftRecordStationKvBeans(StationRequestBean reqBean) {
        // 获取班次信息 redis
        List<StationKvBean> prodShiftDataContext = productionCustomContextStepService.getProdShiftDataContext(reqBean.getOrganizeCode(), reqBean.getWorkCenterCode());
        if (CollectionUtils.isEmpty(prodShiftDataContext)) {
            MesProdShiftRecord shiftRecord = new MesProdShiftRecord();
            shiftRecord.setWorkCenterCode(reqBean.getWorkCenterCode());
            shiftRecord.setWorkStatus(MesExtEnumUtil.WORK_STATUS.START.getValue());
            // 查询数据库
            MesProdShiftRecord prodShiftRecord = mesProdShiftRecordService.queryMesProdShiftRecord(shiftRecord);
            if (prodShiftRecord != null) {
                prodShiftDataContext = getProdShiftData(prodShiftRecord);
                productionCustomContextStepService.dispatchProdShiftDataContext(reqBean.getOrganizeCode(), reqBean.getWorkCenterCode(),prodShiftDataContext);
            }
        }
        return prodShiftDataContext;
    }

    private StationResultBean getStationResultBean(StationRequestBean reqBean, List<List<StationKvBean>> moduleContentContext, List<StationKvBean> prodShiftDataContext) {
        StationResultBean resultBean = new StationResultBean();
        resultBean.setBusiType(MesPcnEnumUtil.STATION_BUSI_TYPE.MODULE_CONTENT.getValue());
        resultBean.setDataType(MesPcnEnumUtil.STATION_DATA_TYPE.CUSTOM.getValue());
        resultBean.setCustomPageName(MesPcnExtConstWords.CUSTOM_PAGE_NAME_DEFAULT);
        resultBean.setSpecialDisplayData(getStepColIndent(reqBean));


        resultBean.setResultObj(prodShiftDataContext);
        resultBean.setResultList(moduleContentContext);
        return resultBean;
    }

    //封装展示组件班组班次内容
    private List<StationKvBean> getProdShiftData(MesProdShiftRecord record) {

        return  StationKvBeanUtil.addStationKvBeanList(new ArrayList<>(),
                new StationKvBean(MesPcnExtConstWords.SHIFT_GROUP, "班组", record.getShiftGroup()),
                new StationKvBean(MesPcnExtConstWords.SHIFT_GROUP_NAME, "班组名称", record.getShiftGroupName()),
                new StationKvBean(MesPcnExtConstWords.SHIFT_CODE, "班次", record.getShiftCode()),
                new StationKvBean(MesPcnExtConstWords.SHIFT_NAME, "班次名称", record.getShiftName()));
    }
}
